from typing import Callable, Awaitable, Optional

class Command:
    async def on_command_response(self, request_id: str, body: dict):
        pass

    async def execute_command(self, command_line: str, call_back: Optional[Callable[[], Awaitable[None]]]):
        pass



class Event:
    AdditionalContentLoaded = 'AdditionalContentLoaded'
    AgentCommand = 'AgentCommand'
    AgentCreated = 'AgentCreated'
    ApiInit = 'ApiInit'
    AppPaused = 'AppPaused'
    AppResumed = 'AppResumed'
    AppSuspended = 'AppSuspended'
    AwardAchievement = 'AwardAchievement'
    BlockBroken = 'BlockBroken'
    BlockPlaced = 'BlockPlaced'
    BoardTextUpdated = 'BoardTextUpdated'
    BossKilled = 'BossKilled'
    CameraUsed = 'CameraUsed'
    CauldronUsed = 'CauldronUsed'
    ChunkChanged = 'ChunkChanged'
    ChunkLoaded = 'ChunkLoaded'
    ChunkUnloaded = 'ChunkUnloaded'
    ConfigurationChanged = 'ConfigurationChanged'
    ConnectionFailed = 'ConnectionFailed'
    CraftingSessionCompleted = 'CraftingSessionCompleted'
    EndOfDay = 'EndOfDay'
    EntitySpawned = 'EntitySpawned'
    FileTransmissionCancelled = 'FileTransmissionCancelled'
    FileTransmissionCompleted = 'FileTransmissionCompleted'
    FileTransmissionStarted = 'FileTransmissionStarted'
    FirstTimeClientOpen = 'FirstTimeClientOpen'
    FocusGained = 'FocusGained'
    FocusLost = 'FocusLost'
    GameSessionComplete = 'GameSessionComplete'
    GameSessionStart = 'GameSessionStart'
    HardwareInfo = 'HardwareInfo'
    HasNewContent = 'HasNewContent'
    ItemAcquired = 'ItemAcquired'
    ItemCrafted = 'ItemCrafted'
    ItemDestroyed = 'ItemDestroyed'
    ItemDropped = 'ItemDropped'
    ItemEnchanted = 'ItemEnchanted'
    ItemSmelted = 'ItemSmelted'
    ItemUsed = 'ItemUsed'
    JoinCanceled = 'JoinCanceled'
    JukeboxUsed = 'JukeboxUsed'
    LicenseCensus = 'LicenseCensus'
    MascotCreated = 'MascotCreated'
    MenuShown = 'MenuShown'
    MobInteracted = 'MobInteracted'
    MobKilled = 'MobKilled'
    MultiplayerConnectionStateChanged = 'MultiplayerConnectionStateChanged'
    MultiplayerRoundEnd = 'MultiplayerRoundEnd'
    MultiplayerRoundStart = 'MultiplayerRoundStart'
    NpcPropertiesUpdated = 'NpcPropertiesUpdated'
    OptionsUpdated = 'OptionsUpdated'
    performanceMetrics = 'performanceMetrics'
    PackImportStage = 'PackImportStage'
    PlayerBounced = 'PlayerBounced'
    PlayerDied = 'PlayerDied'
    PlayerJoin = 'PlayerJoin'
    PlayerLeave = 'PlayerLeave'
    PlayerMessage = 'PlayerMessage'
    PlayerTeleported = 'PlayerTeleported'
    PlayerTransform = 'PlayerTransform'
    PlayerTravelled = 'PlayerTravelled'
    PortalBuilt = 'PortalBuilt'
    PortalUsed = 'PortalUsed'
    PortfolioExported = 'PortfolioExported'
    PotionBrewed = 'PotionBrewed'
    PurchaseAttempt = 'PurchaseAttempt'
    PurchaseResolved = 'PurchaseResolved'
    RegionalPopup = 'RegionalPopup'
    RespondedToAcceptContent = 'RespondedToAcceptContent'
    ScreenChanged = 'ScreenChanged'
    ScreenHeartbeat = 'ScreenHeartbeat'
    SignInToEdu = 'SignInToEdu'
    SignInToXboxLive = 'SignInToXboxLive'
    SignOutOfXboxLive = 'SignOutOfXboxLive'
    SpecialMobBuilt = 'SpecialMobBuilt'
    StartClient = 'StartClient'
    StartWorld = 'StartWorld'
    TextToSpeechToggled = 'TextToSpeechToggled'
    UgcDownloadCompleted = 'UgcDownloadCompleted'
    UgcDownloadStarted = 'UgcDownloadStarted'
    UploadSkin = 'UploadSkin'
    VehicleExited = 'VehicleExited'
    WorldExported = 'WorldExported'
    WorldFilesListed = 'WorldFilesListed'
    WorldGenerated = 'WorldGenerated'
    WorldLoaded = 'WorldLoaded'
    WorldUnloaded = 'WorldUnloaded'

    events = list()

    async def on_event(self, event: str, properties: dict):
        pass

    def has_event(self, event: str) -> bool:
        if event in self.events:
            return True
        return False

    def add_event(self, event: str):
        self.events.append(event)

    def remove_event(self, event: str):
        self.events.remove(event)



from .server import Server
from .event import Event
from .command import Command
from .packet import Packet
from .agent import Agent
from .utils import Utils
import json, uuid
from typing import Callable, Awaitable, Optional

class MinecraftWebSocket(Server, Event, Command, Agent):
    command_responses = list()

    async def on_data(self, data: str):
        data = json.loads(data)
        header = data['header']
        body = data['body']
        messagePurpose = header['messagePurpose']
        if messagePurpose == Packet.Purpose.Event:
            eventName = body['eventName']
            properties = body['properties']
            await self.on_event(eventName, properties)
        elif messagePurpose == Packet.Purpose.CommandResponse:
            request_id = header['requestId']
            await self.on_command_response(request_id, body)

    def reset(self):
        super().reset()
        self.events = list()
        self.command_responses = list()

    async def add_event(self, event: str):
        if not self.has_event(event):
            super().add_event(event)
            packet = Packet(str(uuid.uuid4()), Packet.Purpose.Subscribe, eventName=event)
            await self.send_data(packet.encode())

    async def remove_event(self, event: str):
        if self.has_event(event):
            super().remove_event(event)
            packet = Packet(str(uuid.uuid4()), Packet.Purpose.Unsubscribe, eventName=event)
            await self.send_data(packet.encode())

    async def on_command_response(self, request_id: str, body: dict):
        for command_response in self.command_responses:
            if request_id == command_response[0]:
                await command_response[1](body)
                self.command_responses.remove(command_response)

    async def execute_command(self, command_line: str, call_back: Optional[Callable[[], Awaitable[None]]]):
        request_id = str(uuid.uuid4())
        packet = Packet(request_id, Packet.Purpose.CommandRequest, commandLine=command_line)
        await self.send_data(packet.encode())
        if call_back != None:
            self.command_responses.append([request_id, call_back])



import json

class Packet:
    class Purpose:
        Subscribe = 'subscribe'
        Unsubscribe = 'unsubscribe'
        Event = 'event'
        Error = 'error'
        CommandRequest = 'commandRequest'
        CommandResponse = 'commandResponse'

    VERSION = 1
    body = dict()

    def __init__(self, request_id: str, purpose: str, **kwargs):
        self.request_id = request_id
        self.purpose = purpose
        for key, value in kwargs.items():
            self.body[key] = value

    def encode(self) -> str:
        packet = {
            'header': {
                'requestId': self.request_id,
                'messagePurpose': self.purpose,
                'version': self.VERSION
            },
            'body': self.body
        }
        return json.dumps(packet)



import websockets, asyncio

class Server:
    def __init__(self, host: str = 'localhost', port: int = 8080):
        self.host = host
        self.port = port

    def run(self):
        self.server = websockets.serve(self.__connect, self.host, self.port)
        asyncio.get_event_loop().run_until_complete(self.server)
        asyncio.get_event_loop().run_forever()

    async def __connect(self, conn, _):
        self.conn = conn
        await self.on_connect()
        await self.__recv()

    async def on_connect(self):
        pass

    async def on_disconnect(self):
        pass

    async def on_data(self, data: str):
        pass

    async def __recv(self):
        while True:
            try:
                data = await self.conn.recv()
                await self.on_data(data)
            except websockets.ConnectionClosed:
                await self.on_disconnect()
                self.reset()
                break

    def reset(self):
        self.conn = None

    async def send_data(self, data: str):
        await self.conn.send(data)



from setuptools import *

setup(
    name='OptiCon',
    version='0.0.1',
    packages=['OptiCon'],
    url='https://github.com/IndigoMod/OptiCon',
    license='MIT',
    author='beta5051',
    author_email='AcaiBerii@protonmail.com',
    description='MinecraftWS fork that provides optimal performance and compatibility with IndigoMod.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=['websockets>=8.1']
)



from .minecraft_websocket import MinecraftWebSocket
from .server import Server
from .event import Event
from .command import Command
from .packet import Packet
from .agent import Agent
from .utils import Utils



from .packet import Packet
import json


class Utils:
    def get_json_value(content, name):
        return content[name]

    def decode_json(itm):
        return json.JSONDecoder.decode(itm)

    def encode_json(itm):
        return json.JSONEncoder.encode(itm)
